#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from dateutil import parser
from elasticsearch_dsl import Search
from pyramid.decorator import reify
from pyramid_es import get_client
from z3c.form.button import Button, Buttons
from z3c.form.field import Fields
from zope.interface import Interface, implementer
from zope.schema import TextLine, Datetime
from zope.schema.fieldproperty import FieldProperty

from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content_es.interfaces import IDocumentIndexTarget
from pyams_form.form import ajax_config
from pyams_form.help import FormHelp
from pyams_form.interfaces.form import IFormHelp
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflow
from pyams_zmi.form import AdminDialogEditForm
from pyams_zmi.layer import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_content_es import _


@viewlet_config(name='check-index.menu',
                context=IDocumentIndexTarget, layer=IAdminLayer, view=Interface,
                manager=IContextActions, weight=30,
                permission=MANAGE_CONTENT_PERMISSION)
class ContentIndexCheckerMenu(ToolbarMenuItem):
    """Content index checker menu"""

    label = _("Check index content...")
    label_css_class = 'fa fa-fw fa-check'

    url = 'check-index.html'
    modal_target = True


class IContentIndexCheckerFields(Interface):
    """Content index checker content interface"""

    id = TextLine(title=_("Internal ID"),
                  readonly=True)
    modified = TextLine(title=_("Modification date"),
                        readonly=True)
    status = TextLine(title=_("Workflow status"),
                      readonly=True)


@implementer(IContentIndexCheckerFields)
class ContentIndexCheckerFields:
    """Content index checker fields container"""
    id = FieldProperty(IContentIndexCheckerFields['id'])
    modified = FieldProperty(IContentIndexCheckerFields['modified'])
    status = FieldProperty(IContentIndexCheckerFields['status'])

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)


@adapter_config(context=(IDocumentIndexTarget, Interface),
                provides=IContentIndexCheckerFields)
def document_index_checker_fields_factory(context, request):
    """Document index checker fields factory"""
    translate = request.localizer.translate
    client = get_client(request)
    query = Search().using(client.es) \
                    .index(client.index) \
                    .query('term', **{'_id': context.id}) \
                    .source(['_id', 'workflow'])
    for result in query.execute().hits:
        modified_date = tztime(parser.parse(result.workflow.modified_date))
        workflow = IWorkflow(context)
        label = workflow.get_state_label(result.workflow.status)
        return ContentIndexCheckerFields(id=result.meta.id,
                                         modified=format_datetime(modified_date),
                                         status=translate(label))
    return ContentIndexCheckerFields(id=context.id,
                                     modified='--',
                                     status=translate(_("unknown")))


class IContentIndexCheckerButtons(Interface):
    """Content index checker buttons"""

    close = CloseButton(name='close', title=_("Cancel"))
    reindex = Button(name='reindex', title=_("Force reindex"))


@pagelet_config(name='check-index.html',
                context=IDocumentIndexTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
@ajax_config(name='check-index.json',
             context=IDocumentIndexTarget, layer=IPyAMSLayer)
class ContentIndexCheckerForm(AdminDialogEditForm):
    """Content index checker form"""

    legend = _("Content index checker")
    icon_css_class = 'fa fa-fw fa-check'

    fields = Fields(IContentIndexCheckerFields)
    buttons = Buttons(IContentIndexCheckerButtons)

    edit_permission = MANAGE_CONTENT_PERMISSION

    @reify
    def index_content(self):
        return self.request.registry.queryMultiAdapter((self.context, self.request),
                                                       IContentIndexCheckerFields)

    def getContent(self):
        return self.index_content

    def updateActions(self):
        super().updateActions()
        if 'reindex' in self.actions:
            self.actions['reindex'].addClass('btn-primary')

    def applyChanges(self, data):
        es_client = self.request.registry.pyramid_es_client
        es_client.index_object(self.context)

    def get_ajax_output(self, changes):
        translate = self.request.localizer.translate
        return {
            'status': 'success',
            'message': translate(_("Request successful, the content index has been updated..."))
        }


@adapter_config(context=(Interface, IAdminLayer, ContentIndexCheckerForm),
                provides=IFormHelp)
class ContentIndexCheckerFormHelp(FormHelp):
    """Content index checker form help adapter"""

    message = _("Anormal website behaviour after content publication can come from incorrect "
                "Elasticsearch index content.\n"
                "\n"
                "If you notice such behaviour, you can try to force index update to fix the "
                "problem...")
    message_format = 'rest'
